## Copyright (c) 2005-2006, Gillmer J. Derge.

## This file is part of Civilization IV Warlords ViSa Mod
##
## Civilization IV Warlords ViSa Mod is free software; you can redistribute
## it and/or modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation; either
## version 2 of the License, or (at your option) any later version.
##
## Civilization IV Warlords ViSa Mod is distributed in the hope that it will
## be useful, but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
## See the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Civilization IV Warlords ViSa Mod; if not, write to the Free
## Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
## 02110-1301 USA

# edit by Keldath & Mrgenie & TAfirehawk
	
from CvPythonExtensions import *
import CvUtil

import CvEventManager
import CvUnitAllegianceEventManager
import CvUnitStatisticsEventManager
import CvCulturalInfluencesEventManager
import CvCulturalDecayEventManager
import CvEnhancedCultureConquestEventManager
import CvGreatPersonModEventManager
import CvEnhancedTechConquestEventManager
import CvGDEventManager
import CvFieldHospitalEventManager
import CvGreatStatesmanEventManager
import CvJZoCEventManager
import RealFort
import ViSa_Ini_Tweaks
import CvGreatGeneralsFromBarbarianCombatEventManager

## OOS Logging for tracking MP problems
import OOSLogger
## end of OOS logging, thanks to gerikes for supporting on this.

## Partisan begin (1/3)
import sys
import PyHelpers
import CvCameraControls
import Partisan
## Partisan end (1/3)

## techleak begin (1/5)
import ViSa_TechLeak
## techleak begin (1/5)

##imported (1/2) from mercenaries mod, source: http://forums.civfanatics.com/showthread.php?p=3779841
#import CvMercEventManager
##end of Merc (1/2)

## begin GIR ADD on (1/6)
import Monument
## end GIR ADD on (1/6)



gc = CyGlobalContext()

## Partisan begin (2/3)
Partisan = Partisan.adPartisan()
## Partisan end (2/3)

realFort = RealFort.RealFort()

## begin GIR ADD ON (2/6)
monumentImprovement = Monument.MonumentImprovement()
monumentBuilding = Monument.MonumentBuilding()
monumentGoldenAge = Monument.MonumentGoldenAge()
monumentNSFTC = Monument.TrainingNSFTC()
monumentCrusade = Monument.Monument_onBeginPlayerTurn()
CMI = CvEventManager.CvScreensInterface.CvMainInterface
## end GIR ADD ON (2/6)


## Techleak begin (2/5)
TechLeak = ViSa_TechLeak.TechLeak()
## Techleak end (2/5)

PyPlayer = PyHelpers.PyPlayer


class CvViSaEventManager(CvEventManager.CvEventManager, object):

	CustomEvents = {}
		
	def __init__(self, *args, **kwargs):           
		## initialize base class partisan mod
		self.parent = CvEventManager.CvEventManager
		self.parent.__init__(self)
		## end initialize partisan
            		   
		super(CvViSaEventManager, self).__init__(*args, **kwargs)
		# map the initial EventHandlerMap values into the new data structure
		for eventType, eventHandler in self.EventHandlerMap.iteritems():
			self.setEventHandler(eventType, eventHandler)
		
		# --> INSERT EVENT HANDLER INITIALIZATION HERE <--
		CvGreatPersonModEventManager.CvGreatPersonModEventManager(self)
		CvUnitAllegianceEventManager.CvUnitAllegianceEventManager(self)
		CvUnitStatisticsEventManager.CvUnitStatisticsEventManager(self)
		CvCulturalInfluencesEventManager.CvCulturalInfluencesEventManager(self)
		CvCulturalDecayEventManager.CvCulturalDecayEventManager(self)
		CvEnhancedCultureConquestEventManager.CvEnhancedCultureConquestEventManager(self)
		CvEnhancedTechConquestEventManager.CvEnhancedTechConquestEventManager(self)
		CvGDEventManager.CvGDEventManager(self)
		CvFieldHospitalEventManager.CvFieldHospitalEventManager(self)
		CvGreatStatesmanEventManager.CvGreatStatesmanEventManager(self)
		CvJZoCEventManager.CvJZoCEventManager(self)
		ViSa_Ini_Tweaks.CvVisaIniTweaks(self)
		CvGreatGeneralsFromBarbarianCombatEventManager.CvGreatGeneralsFromBarbarianCombatEventManager(self)
		## begin merc mod (2/2)
		#CvMercEventManager.CvMercEventManager(self)
		## end merc mod (2/2)

	
	def onGameUpdate(self, argsList):
		'sample generic event, called on each game turn slice'
		genericArgs = argsList[0][0]	# tuple of tuple of my args
		turnSlice = genericArgs[0]

		# Added by Gerikes for OOS logging
		OOSLogger.doGameUpdate()
		# End added by Gerikes for OOS logging.

	def onCityAcquired(self, argsList):
		### Partisan anfang (3/3) ###
		Partisan.onCityAcquired(argsList)
		### Partisan ende (3/3) ###
		self.parent.onCityAcquired(self, argsList)		
								

	def beginEvent( self, context, argsList=-1 ):
		"Begin Event"
		if(self.CustomEvents.has_key(context)):
			return self.CustomEvents[context][2](argsList)
		else:
			return CvEventManager.CvEventManager.beginEvent(self, context, argsList)
			
	def applyEvent( self, argsList ):
		'Apply the effects of an event '
		context, playerID, netUserData, popupReturn = argsList
		
		if(self.CustomEvents.has_key(context)):
			entry = self.CustomEvents[context]
			# the apply function
			return entry[1]( playerID, netUserData, popupReturn )   
		else:
			return CvEventManager.CvEventManager.applyEvent(self, argsList)


	def addCustomEventDefinition(self, eventType, eventDefinition):
		self.CustomEvents[eventType] = eventDefinition

	def removeCustomEventDefinition(self, eventType):
		del self.CustomEvents[eventType]

	def setCustomEventDefinition(self, eventType, eventDefinition):
		self.CustomEvents[eventType] = eventDefinition

	def addEventHandler(self, eventType, eventHandler):
		"""Adds a handler for the given event type.
		
		A list of supported event types can be found in the initialization 
		of EventHandlerMap in the CvEventManager class.

		"""
		self.EventHandlerMap[eventType].append(eventHandler)

	def removeEventHandler(self, eventType, eventHandler):
		"""Removes a handler for the given event type.
		
		A list of supported event types can be found in the initialization 
		of EventHandlerMap in the CvEventManager class.  It is an error if 
		the given handler is not found in the list of installed handlers.

		"""
		self.EventHandlerMap[eventType].remove(eventHandler)
	
	def setEventHandler(self, eventType, eventHandler):
		"""Removes all previously installed event handlers for the given 
		event type and installs a new handler .
		
		A list of supported event types can be found in the initialization 
		of EventHandlerMap in the CvEventManager class.  This method is 
		primarily useful for overriding, rather than extending, the default 
		event handler functionality.

		"""
		self.EventHandlerMap[eventType] = [eventHandler]

	def handleEvent(self, argsList):
		"""Handles events by calling all installed handlers."""
		self.origArgsList = argsList
		flagsIndex = len(argsList) - 6
		self.bDbg, self.bMultiPlayer, self.bAlt, self.bCtrl, self.bShift, self.bAllowCheats = argsList[flagsIndex:]		
		eventType = argsList[0]
		return {
			"kbdEvent": self._handleConsumableEvent,
			"mouseEvent": self._handleConsumableEvent,
			"OnSave": self._handleOnSaveEvent,
			"OnLoad": self._handleOnLoadEvent
		}.get(eventType, self._handleDefaultEvent)(eventType, argsList[1:])

	def _handleDefaultEvent(self, eventType, argsList):
		if self.EventHandlerMap.has_key(eventType):
			for eventHandler in self.EventHandlerMap[eventType]:
				# the last 6 arguments are for internal use by handleEvent
				eventHandler(argsList[:len(argsList) - 6])

	def _handleConsumableEvent(self, eventType, argsList):
		"""Handles events that can be consumed by the handlers, such as
		keyboard or mouse events.
		
		If a handler returns non-zero, processing is terminated, and no 
		subsequent handlers are invoked.

		"""
		if self.EventHandlerMap.has_key(eventType):
			for eventHandler in self.EventHandlerMap[eventType]:
				# the last 6 arguments are for internal use by handleEvent
				result = eventHandler(argsList[:len(argsList) - 6])
				if (result > 0):
					return result
		return 0

	# TODO: this probably needs to be more complex
	def _handleOnSaveEvent(self, eventType, argsList):
		"""Handles OnSave events by concatenating the results obtained
		from each handler to form an overall consolidated save string.

		"""
		result = ""
		if self.EventHandlerMap.has_key(eventType):
			for eventHandler in self.EventHandlerMap[eventType]:
				# the last 6 arguments are for internal use by handleEvent
				result = result + eventHandler(argsList[:len(argsList) - 6])
		return result

	# TODO: this probably needs to be more complex
	def _handleOnLoadEvent(self, eventType, argsList):
		"""Handles OnLoad events."""
		return self._handleDefaultEvent(eventType, argsList)
###########----------------------------------##############
                        ## ADDS by Mrgenie##
###########----------------------------------##############

	def onLoadGame(self, argsList):
		self.parent.onLoadGame(self, argsList)
		## techleak begin (3/5)
		TechLeak.__init__()
		## techleak begin (3/5)
		

	def onBeginGameTurn(self, argsList):
		## begin GIR ADD ON (3/7)
		self.parent.onBeginGameTurn(self, argsList)
		monumentNSFTC.onBeginGameTurn(argsList)
		## end GIR ADD ON (3/7)		

			
	def onBeginPlayerTurn(self, argsList):
		## techleak begin (4/5)
		TechLeak.onBeginPlayerTurn(argsList)
		TechLeak.onTechLeakFirst(argsList)
		## techleak begin (4/5)
		##begin GIR ADD ON (4/7)
		monumentCrusade.onBeginPlayerTurn(argsList)
		self.parent.onBeginPlayerTurn(self, argsList)
		## end GIR ADD ON (4/7)


	def onEndPlayerTurn(self, argsList):
		## techleak begin (5/5)
		iGameTurn, iPlayer = argsList
		TechLeak.onEndPlayerTurn(iPlayer)
		self.parent.onEndPlayerTurn
		## techleak begin (5/5)
		

	def onImprovementBuilt(self, argsList):
		## begin GIR ADD ON (5/7)
		self.parent.onImprovementBuilt(self, argsList)
		monumentImprovement.onImprovementBuilt(argsList)		
		realFort.onImprovementBuilt(argsList)
		## end GIR ADD ON (5/7)
		

	def onUnitBuilt(self, argsList):
		## begin GIR ADD ON (6/7)
		self.parent.onUnitBuilt(self, argsList)
		monumentGoldenAge.onUnitBuilt(argsList)
		## end GIR ADD ON (6/7)

	def onBuildingBuilt(self, argsList):
		## begin GIR ADD ON (7/7)
		self.parent.onBuildingBuilt(self, argsList)
		monumentBuilding.onBuildingBuilt(argsList)
		## end GIR ADD ON (7/7)
		
	def onUnitMove(self, argsList):
		self.parent.onUnitMove(self, argsList);
		realFort.onUnitMove(argsList)
		
	def onTechAcquired(self, argsList):
		self.parent.onTechAcquired(self, argsList)
		realFort.onTechAcquired(argsList)	
				
				
		
	


